<template>
    <div class="chart-container" :class="{'maxSize': maxSize}" ref="root" :style="getStyle">
        <div class="chart-content-container"
        :class="{'chart-content-fixed':isFull}">
            <div class="chart-content" :id="id" :style="getStyle"></div>
        </div>
    </div>
</template>
<script>
// 引入 ECharts 主模块
import ECharts from "echarts/lib/echarts";
//引入折线图
import "echarts/lib/chart/line";
//引入提示框
import "echarts/lib/component/tooltip";
//引入标题
import "echarts/lib/component/title";
//引入图例标志
import "echarts/lib/component/legend";
//区域缩放
import "echarts/lib/component/dataZoom";
// 引入自定义主题
import "./theme/transparent"

let resizeHandler;
let chart = null;
let g_opt = null;
export default {
    props: {
        title: {
            type: String,
            default: ''
        },
        height: {
            type: [Number, String],
            default: 300
        },
        // width: {
        //     type: [Number, String],
        //     default: 300
        // },
        maxSize: {
            type: Boolean,
            default: false
        },
        id: {
            type: String,
            default: ''
        },
        'show-zoom': {
            type: Boolean,
            default: false,
        },
        zoom: {
            type: Boolean,
            default: true,
        },
        end: {
            type: Number,
            default: 100,
        },
        subtext: {
            type: Boolean,
            default: false,
        },
        'trigger-on': {
            type: String,
            default: 'mousemove',
        },
        delay: {
            type: Number,
            default: 3000,
        }
        ,categoryLen: {
            type: Number,
            default: 1
        }
        ,showXaxis: {
            type: Boolean,
            default: false
        }
        ,subTitleWidth: [String, Number]
        // 标题（属性）过长 做切分换行处理 此处切分片断的长度 默认20 少于20的不会切分
        ,titleSplitLen: {
            type: Number,
            default: 20
        }
    },
    data() {
        return {
            chart: "",
            tooltip: {
                triggerOn: this.triggerOn,
            },
            dataZoom: {
                show: this.showZoom,
                disabled: this.zoom,
                start: 0,
                end: this.end,
            },
            // itemHeight: 40,
            isFull: false,
            dataIndex: -1,
            unit: []
        }
    },
    methods: {
        setOption(opt) {
            // console.trace('lineChart setOption');
            // console.log(opt);
            let dataZoom = this.dataZoom;
            // let tooltip = this.tooltip;
            const self = this;
            let rootH = this.$refs.root.offsetHeight;
            g_opt = opt;
            // console.log(this.$refs.root.clientHeight, this.$refs.root.offsetHeight, 99999999988888);
            if (this.showZoom) {
                rootH -= 30;
            }
            if (this.showXaxis) {
                rootH -= 30;
            }
            const itemHeight = rootH / this.categoryLen - 2;
            // console.log(rootH, itemHeight, '??');

            // console.log(this.$refs.root.offsetHeight);
            // 默认配置项
            let option = {
                tooltip: {
                    trigger: 'axis'
                    ,confine: true
                    ,hideDelay: 500
                    // ,alwaysShowContent: true
                    ,formatter: function(params) {
                        var res = params[0].name;
                        // var cols = Math.ceil(params.length/5);
                        for(var i=0; i<params.length; i++) {
                            /*if(i%cols == 0) {
                                res += '<br>';
                            }*/
                            res += '<br>' + params[i].marker+params[i].seriesName
                            +': '+params[i].value + '&ensp;' + self.unit[params[i].axisIndex]
                            "<span style='display: inline-block;margin-right: 8px'></span>";
                        }
                        return res;
                    }
                },
                
                xAxis: {
                    show: false,
                    data: []
                },
                yAxis: {
                    show: false
                },
                axisPointer: {
                    snap: true,
                    // triggerOn: 'mousemove|click',
                    // triggerOn: 'none',
                    link: {
                        xAxisIndex: 'all'
                    }
                }
            };

            let xLabel = [];
            if (opt.xLabel) {
                xLabel = opt.xLabel;
            }

            if (opt.series && opt.series.length) {
                let title = [];
                let grid = [];
                let xAxis = [];
                let yAxis = [];
                let series = [];
                let xAxisIndex = [];
                const reg = new RegExp('(\\S{' + this.titleSplitLen + '})', 'g');
                opt.series.forEach((v, i, a) => {
                    /*let subtext = xLabel.length ? ('最新数据: ' + xLabel[xLabel.length - 1] + ' '  + v.data[v.data.length - 1] + v.unit)
                        : '';*/
                    // 生成 title
                    title.push({
                        left: 'left',
                        text: v.name.length > this.titleSplitLen ? v.name.replace(reg, '$1\n') : v.name,
                        // subtext: subtext,
                        textStyle: {
                            fontSize: 12
                        },
                        top: itemHeight * i + itemHeight / 2 - 12
                    });
                    grid.push({
                        left: this.subTitleWidth ? this.subTitleWidth : 80
                        ,right: 40
                        ,top: itemHeight * i
                        ,height: itemHeight
                    });
                    xAxis.push({
                        show: i == a.length - 1
                        // show: true
                        ,type: 'category'
                        ,gridIndex: i
                        ,data: xLabel
                        ,boundaryGap: false
                    });
                    yAxis.push({
                        show: false
                        ,gridIndex: i
                        ,splitLine: {
                            show: false
                        }
                        ,max: (v) => {
                            return v.max * 1.08;
                        }
                    });
                    series.push({
                        type: 'line'
                        ,name: v.name
                        ,smooth: !v.step
                        ,symbolSize: 0
                        ,sampling: 'average'
                        ,hoverAnimation: false
                        ,data: v.data
                        ,xAxisIndex: i
                        ,step: !!v.step
                        ,yAxisIndex: i
                    });
                    this.unit.push(v.unit);
                    xAxisIndex.push(i);
                });
                // console.log(xAxis, 'xAxis');
                option.title = title;
                option.grid = grid;
                option.xAxis = xAxis;
                option.yAxis = yAxis;
                option.series = series;
                option.dataZoom = [{
                    show: dataZoom.show,
                    start: 0,
                    end: dataZoom.end,
                    xAxisIndex: xAxisIndex,
                    realtime: false,
                    bottom: '0',
                    left: '20',
                    right: '20px',
                    fillerColor: 'rgba(167,183,204,0.1)',
                    dataBackground: {
                        lineStyle: {
                            color: '#15E3F3'
                        },
                        areaStyle: {
                            color: '#15E3F3',
                            opacity: 0,
                        }
                    },
                }];
            }


            // 绘制图表
            this.$G.chartManage.get(this.id).setOption(option);
            // console.log(this.$G.chartManage.get(this.id).getOption(), 'options');
            // 返回chart实例
            return this.$G.chartManage.get(this.id);
        }
    },
    computed: {
        getStyle(){
            let height = this.height;
            // let width = this.width;
            if(typeof height == 'number') {
                height += 'px';
            }
            // if(typeof width == 'number') {
            //     width += 'px';
            // }
            return {
                height: height,
                // width: width
            }
        }
        ,setCanvasHeight () {
            return {
                height: this.itemHeight * this.categoryLen + 'px'
            }
        }
    },
    mounted() {
        var ele = document.getElementById(this.id);
        chart = ECharts.init(ele, 'transparent');
        // 将图表添加到图表管理
        this.$G.chartManage.set(this.id, chart);
        // 设置配置
        this.setOption({});
        resizeHandler = () => {
            if (this.$G.chartManage.get(this.id)) {
                this.$G.chartManage.get(this.id).resize();
            }
        }
        window.addEventListener('resize', resizeHandler);
        // 滚动事件
        let scrollTimeStamp = 0;
        // let fn = this.$common.throttle(800);
        this.$refs.root.onscroll = (e) => {
            /*fn(() => {
                console.log('scroll', e.timeStamp);
            }, e.timeStamp);*/
        }

    },
    updated () {
        // let h = this.$refs.root.currentStyle || window.getComputedStyle(this.$refs.root, null);
        // console.log('update', h.height);
        this.$nextTick(() => {
            // console.log('resize', h.height);
            this.setOption(g_opt).resize();
        });
        // debugger;
    },
    destroyed() {
        // 销毁echarts
        this.$G.chartManage.del(this.id);
        window.removeEventListener('resize', resizeHandler);
    }
}
</script>

<style scoped>
.maxSize {
    width: 100%;
    height: 100%;
}
.chart-content-container,
.chart-content {
    height: 100%;
}

.chart-content-container {
    position: relative;
}
.chart-content-container.chart-content-fixed {
    position: fixed;
    top: 0;
    left: 0;
    bottom: 0;
    right: 0;
    z-index: 99999999;
    background: url('../../assets/images/dw_bg.png') no-repeat;
	background-size: 100% 100%;
}
.chart-tools {
    position: absolute;
    top: 0.2rem;
    right: 0.2rem;
    z-index: 99;
}
.chart-content-container.chart-content-fixed .chart-tools {
    z-index: 99999999;
}
.chart-tools .iconfont {
    font-size: 0.22rem;
    margin-right: 0.1rem;
}
.chart-tools .iconfont:hover {
    color: #b4afaf;
}
.chart-tools .iconfont:active {
    color: #FF0000;
}
</style>


